@@ -52,6 +52,7 @@ extern "C" {
 #include "ptable.h"
 #include "srl_buffer.h"
 #include "srl_compress.h"
+#include "qsort.h"
 
 /* The ENABLE_DANGEROUS_HACKS (passed through from ENV via Makefile.PL) enables
  * optimizations that may make the code so cozy with a particular version of the
@@ -1133,220 +1134,204 @@ srl_dump_av(pTHX_ srl_encoder_t *enc, AV *src, U32 refcount)
 
 #define SIGN(cmp) (cmp < 0 ? -1 : cmp == 0 ? 0 : 1)
 
-/* compare hash entries, used when all keys are bytestrings */
-static int
-he_cmp_fast(const void *a_, const void *b_)
+SRL_STATIC_INLINE int
+he_cmp_fast_inline(HE *a, HE *b)
 {
-    /* even though we are called as a callback from qsort there is
-     * no need for a dTHX here, we don't use anything that needs it */
-    HE *a = *(HE **)a_;
-    HE *b = *(HE **)b_;
-
+    /* no need for a dTHX here, we don't use anything that needs it */
     STRLEN la = HeKLEN(a);
     STRLEN lb = HeKLEN(b);
 
     int cmp = memcmp(HeKEY(a), HeKEY(b), la < lb ? la : lb);
     if (!cmp)
-        cmp = lb - la;
-
-    if (0) warn(
-        "he_cmp_fast: %*s cmp %*s : %d",
-        (int)la, HeKEY(a),
-        (int)lb, HeKEY(b),
-        SIGN(cmp)
-    );
-
+        cmp = la - lb; /* "a" should less than "aa", so 1 - 2 == -1 */
     return cmp;
 }
+#define ISLT_HE_FAST(a,b)   ( he_cmp_fast_inline( *a, *b ) < 0 )
+#define ISLT_HE_SLOW(a,b)   ( sv_cmp( HeSVKEY_force( *a ), HeSVKEY_force( *b ) ) < 0 )
+#define ISLT_KV(a,b)        ( sv_cmp( a->key, b->key ) < 0 )
 
-/* compare hash entries, used when some keys are sv's or utf8 */
-static int
-he_cmp_slow(const void *a, const void *b)
+SRL_STATIC_INLINE void
+srl_dump_hv_unsorted_nomg(pTHX_ srl_encoder_t *enc, HV *src, UV n)
 {
-    /* we are called as a callback from qsort, so no pTHX
-     * is possible in our argument signature, so we need to do a
-     * dTHX; here ourselves. */
-    dTHX;
-    int cmp= sv_cmp( HeSVKEY_force( *(HE **)a), HeSVKEY_force( *(HE **)b ) );
+    HE *he;
+    const int do_share_keys = HvSHAREKEYS((SV *)src);
+    HE **he_ptr= HvARRAY(src);
+    HE **he_end= he_ptr + HvMAX(src) + 1;
 
-    if (0) warn(
-        "he_cmp_slow: %"SVf" cmp %"SVf" : %d",
-        HeSVKEY_force( *(HE **)a ),
-        HeSVKEY_force( *(HE **)b ),
-        SIGN(cmp)
-    );
-
-    return cmp;
+    do {
+        for (he= *he_ptr++; he; he= HeNEXT(he) ) {
+            SV *v= HeVAL(he);
+            if (v != &PL_sv_placeholder) {
+                srl_dump_hk(aTHX_ enc, he, do_share_keys);
+                CALL_SRL_DUMP_SV(enc, v);
+                if (--n == 0) {
+                    he_ptr= he_end;
+                    break;
+                }
+            }
+        }
+    } while ( he_ptr < he_end );
 }
 
-static int
-kv_cmp_slow(const void *a_, const void *b_)
+SRL_STATIC_INLINE void
+srl_dump_hv_unsorted_mg(pTHX_ srl_encoder_t *enc, HV *src, const UV n)
 {
-    /* we are called as a callback from qsort, so no pTHX
-     * is possible in our argument signature, so we need to do a
-     * dTHX; here ourselves. */
-    int cmp;
-    const kv_sv *a = (kv_sv *)a_;
-    const kv_sv *b = (kv_sv *)b_;
-    dTHX;
+    HE *he;
+    UV i= 0;
+    const int do_share_keys = HvSHAREKEYS((SV *)src);
 
-    cmp= sv_cmp( a->key, b->key );
+    (void)hv_iterinit(src); /* return value not reliable according to API docs */
+    while ((he = hv_iternext(src))) {
+        SV *v;
+        if (expect_false( i == n ))
+            croak("Panic: cannot serialize a tied hash which changes its size!");
+        v= hv_iterval(src, he);
+        srl_dump_hk(aTHX_ enc, he, do_share_keys);
+        CALL_SRL_DUMP_SV(enc, v);
+        ++i;
+    }
+    if (expect_false( i != n ))
+        croak("Panic: cannot serialize a tied hash which changes its size!");
+}
 
-    if (0) warn(
-        "kv_cmp_slow: %"SVf" cmp %"SVf": %d",
-        a->key,
-        b->key,
-        SIGN(cmp)
-    );
-    return cmp;
+#define CALL_QSORT_NOBYTES(TYPE, ARY, N, F)                 \
+    STMT_START {                                            \
+        /* hack to forcefully disable "use bytes" */        \
+        COP cop= *PL_curcop;                                \
+        cop.op_private= 0;                                  \
+                                                            \
+        ENTER;                                              \
+        SAVETMPS;                                           \
+                                                            \
+        SAVEVPTR (PL_curcop);                               \
+        PL_curcop= &cop;                                    \
+                                                            \
+        QSORT(TYPE, ARY, N, F)                              \
+                                                            \
+        FREETMPS;                                           \
+        LEAVE;                                              \
+    } STMT_END
+
+SRL_STATIC_INLINE void
+srl_dump_hv_sorted_mg(pTHX_ srl_encoder_t *enc, HV *src, const UV n)
+{
+    HE *he;
+    UV i= 0;
+    const int do_share_keys = HvSHAREKEYS((SV *)src);
+
+    (void)hv_iterinit(src); /* return value not reliable according to API docs */
+    /* can't use the same optimizations we use for normal hashes with tied
+     * hashes - the HE*'s that are returned for tied hashes are temporary
+     * structures returned from the tie infra. So we make an array, and
+     * then sort it.*/
+    {
+        kv_sv *kv_sv_array;
+        kv_sv *kv_sv_array_end;
+        Newx(kv_sv_array, n, kv_sv);
+        SAVEFREEPV(kv_sv_array);
+        kv_sv_array_end= kv_sv_array + n;
+        while ((he = hv_iternext(src))) {
+            if (expect_false( i == n ))
+                croak("Panic: cannot serialize a tied hash which changes its size!");
+            kv_sv_array[i].key= hv_iterkeysv(he);
+            kv_sv_array[i].val= hv_iterval(src,he);
+            i++;
+        }
+        if (expect_false( i != n ))
+            croak("Panic: can not serialize a tied hash which changes it size!");
+
+        CALL_QSORT_NOBYTES(kv_sv, kv_sv_array, n, ISLT_KV);
+
+        while ( kv_sv_array < kv_sv_array_end ) {
+            CALL_SRL_DUMP_SV(enc, kv_sv_array->key);
+            CALL_SRL_DUMP_SV(enc, kv_sv_array->val);
+            kv_sv_array++;
+        }
+    }
+}
+
+
+SRL_STATIC_INLINE void
+srl_dump_hv_sorted_nomg(pTHX_ srl_encoder_t *enc, HV *src, const UV n)
+{
+    HE *he;
+    const int do_share_keys = HvSHAREKEYS((SV *)src);
+
+    (void)hv_iterinit(src); /* return value not reliable according to API docs */
+    {
+        HE **he_array;
+        HE **he_array_ptr;
+        HE **he_array_end;
+        int fast = 1;
+        Newxz(he_array, n, HE*);
+        SAVEFREEPV(he_array);
+        he_array_ptr = he_array;
+        he_array_end = he_array + n;
+        while ((he = hv_iternext(src))) {
+            *he_array_ptr++ = he;
+            if ( HeKLEN(he) < 0 || HeKUTF8(he) )
+                fast = 0;
+        }
+        if (fast) {
+            QSORT(HE *, he_array, n, ISLT_HE_FAST)
+        } else {
+            CALL_QSORT_NOBYTES(HE *, he_array, n, ISLT_HE_SLOW);
+        }
+        while ( he_array < he_array_end ) {
+            SV *v;
+            he = *he_array++;
+            v = hv_iterval(src, he);
+            srl_dump_hk(aTHX_ enc, he, do_share_keys);
+            CALL_SRL_DUMP_SV(enc, v);
+        }
+    }
 }
 
 SRL_STATIC_INLINE void
 srl_dump_hv(pTHX_ srl_encoder_t *enc, HV *src, U32 refcount)
 {
     HE *he;
-    const int do_share_keys = HvSHAREKEYS((SV *)src);
     UV n;
-    U32 dosort= SRL_ENC_HAVE_OPTION(enc, SRL_F_SORT_KEYS);
-
-    if ( dosort || SvMAGICAL(src) ) {
-        UV i= 0;
+    if ( SvMAGICAL(src) ) {
         /* for tied hashes, we have to iterate to find the number of entries. Alas... */
+        n= 0;
         (void)hv_iterinit(src); /* return value not reliable according to API docs */
-        n = 0;
         while ((he = hv_iternext(src))) { ++n; }
-
-        BUF_SIZE_ASSERT_HV(&enc->buf, n);
-
-        if (n < 16 && refcount == 1 && !SRL_ENC_HAVE_OPTION(enc,SRL_F_CANONICAL_REFS)) {
-            enc->buf.pos--; /* back up over the previous REFN */
-            srl_buf_cat_char_nocheck(&enc->buf, SRL_HDR_HASHREF + n);
-        } else {
-            srl_buf_cat_varint_nocheck(aTHX_ &enc->buf, SRL_HDR_HASH, n);
-        }
-
-        (void)hv_iterinit(src); /* return value not reliable according to API docs */
-        if ( dosort ) {
-            /* can't use the same optimizations we use for normal hashes with tied
-             * hashes - the HE*'s that are returned for tied hashes are temporary
-             * structures returned from the tie infra. So we make an array, and
-             * then sort it.*/
-            if ( SvMAGICAL(src) ) {
-                kv_sv *kv_sv_array;
-                kv_sv *kv_sv_array_end;
-                Newx(kv_sv_array, n, kv_sv);
-                SAVEFREEPV(kv_sv_array);
-                kv_sv_array_end= kv_sv_array + n;
-                while ((he = hv_iternext(src))) {
-                    if (expect_false( i == n ))
-                        croak("Panic: cannot serialize a tied hash which changes its size!");
-                    kv_sv_array[i].key= hv_iterkeysv(he);
-                    kv_sv_array[i].val= hv_iterval(src,he);
-                    i++;
-                }
-                if (expect_false( i != n ))
-                    croak("Panic: can not serialize a tied hash which changes it size!");
-
-                {
-                    /* hack to forcefully disable "use bytes" */
-                    COP cop= *PL_curcop;
-                    cop.op_private= 0;
-
-                    ENTER;
-                    SAVETMPS;
-
-                    SAVEVPTR (PL_curcop);
-                    PL_curcop= &cop;
-
-                    qsort(kv_sv_array, n, sizeof(kv_sv), kv_cmp_slow);
-
-                    FREETMPS;
-                    LEAVE;
-                }
-                while ( kv_sv_array < kv_sv_array_end ) {
-                    CALL_SRL_DUMP_SV(enc, kv_sv_array->key);
-                    CALL_SRL_DUMP_SV(enc, kv_sv_array->val);
-                    kv_sv_array++;
-                }
-            } else {
-                HE **he_array;
-                int fast = 1;
-                Newxz(he_array, n, HE*);
-                SAVEFREEPV(he_array);
-                while ((he = hv_iternext(src))) {
-                    he_array[i++]= he;
-                    if (HeKLEN (he) < 0 || HeKUTF8 (he))
-                        fast = 0;
-                }
-                if (fast) {
-                    qsort(he_array, n, sizeof (HE *), he_cmp_fast);
-                } else {
-                    /* hack to forcefully disable "use bytes" */
-                    COP cop= *PL_curcop;
-                    cop.op_private= 0;
-
-                    ENTER;
-                    SAVETMPS;
-
-                    SAVEVPTR (PL_curcop);
-                    PL_curcop= &cop;
-
-                    qsort(he_array, n, sizeof (HE *), he_cmp_slow);
-
-                    FREETMPS;
-                    LEAVE;
-                }
-                for ( i= 0; i < n ; i++ ) {
-                    SV *v;
-                    he= he_array[i];
-                    v= hv_iterval(src, he);
-                    srl_dump_hk(aTHX_ enc, he, do_share_keys);
-                    CALL_SRL_DUMP_SV(enc, v);
-                }
-            }
-        } else {
-            while ((he = hv_iternext(src))) {
-                SV *v;
-                if (expect_false( i == n ))
-                    croak("Panic: cannot serialize a tied hash which changes its size!");
-                v= hv_iterval(src, he);
-                srl_dump_hk(aTHX_ enc, he, do_share_keys);
-                CALL_SRL_DUMP_SV(enc, v);
-                ++i;
-            }
-            if (expect_false( i != n ))
-                croak("Panic: cannot serialize a tied hash which changes its size!");
-        }
-    } else {
+    }
+    else {
         n= HvUSEDKEYS(src);
-        BUF_SIZE_ASSERT_HV(&enc->buf, n);
-        if (n < 16 && refcount == 1 && !SRL_ENC_HAVE_OPTION(enc,SRL_F_CANONICAL_REFS)) {
-            enc->buf.pos--; /* backup over the previous REFN */
-            srl_buf_cat_char_nocheck(&enc->buf, SRL_HDR_HASHREF + n);
-        } else {
-            srl_buf_cat_varint_nocheck(aTHX_ &enc->buf, SRL_HDR_HASH, n);
+    }
+
+    BUF_SIZE_ASSERT_HV(&enc->buf, n);
+    if (n < 16 && refcount == 1 && !SRL_ENC_HAVE_OPTION(enc,SRL_F_CANONICAL_REFS)) {
+        enc->buf.pos--; /* backup over the previous REFN */
+        srl_buf_cat_char_nocheck(&enc->buf, SRL_HDR_HASHREF + n);
+    } else {
+        srl_buf_cat_varint_nocheck(aTHX_ &enc->buf, SRL_HDR_HASH, n);
+    }
+
+    if ( n ) {
+        if ( SvMAGICAL(src) ) {
+            if ( SRL_ENC_HAVE_OPTION(enc, SRL_F_SORT_KEYS) ) {
+                srl_dump_hv_sorted_mg(aTHX_ enc, src, n);
+            }
+            else {
+                srl_dump_hv_unsorted_mg(aTHX_ enc, src, n);
+            }
         }
-        if (n) {
-            HE **he_ptr= HvARRAY(src);
-            HE **he_end= he_ptr + HvMAX(src) + 1;
-            do {
-                for (he= *he_ptr++; he; he= HeNEXT(he) ) {
-                    SV *v= HeVAL(he);
-                    if (v != &PL_sv_placeholder) {
-                        srl_dump_hk(aTHX_ enc, he, do_share_keys);
-                        CALL_SRL_DUMP_SV(enc, v);
-                        if (--n == 0) {
-                            he_ptr= he_end;
-                            break;
-                        }
-                    }
-                }
-            } while ( he_ptr < he_end );
+        else {
+            if ( SRL_ENC_HAVE_OPTION(enc, SRL_F_SORT_KEYS) ) {
+                srl_dump_hv_sorted_nomg(aTHX_ enc, src, n);
+            }
+            else {
+                srl_dump_hv_unsorted_nomg(aTHX_ enc, src, n);
+            }
         }
     }
 }
 
 
+
 SRL_STATIC_INLINE void
 srl_dump_hk(pTHX_ srl_encoder_t *enc, HE *src, const int share_keys)
 {
