@@ -627,18 +627,18 @@ sub have_encoder_and_decoder {
     }
     eval "use $Class; 1"
     or do {
-        note("Could not locate $Class for testing" . ($@ ? " (Exception: $@)" : ""));
+        diag("Could not locate $Class for testing" . ($@ ? " (Exception: $@)" : ""));
         return();
     };
 
     eval "use $need_class; 1"
     or do {
-        note("Could not locate $need_class for testing" . ($@ ? " (Exception: $@)" : ""));
+        diag("Could not locate $need_class for testing" . ($@ ? " (Exception: $@)" : ""));
         return();
     };
     my $cmp_v = $need_class->VERSION;
-    if ($min_v and $cmp_v <= $min_v) {
-        note("Could not load correct version of $need_class for testing "
+    if ($min_v and $cmp_v < $min_v) {
+        diag("Could not load correct version of $need_class for testing "
              ."(got: $cmp_v, needed at least $min_v)");
         return;
     }
@@ -646,7 +646,7 @@ sub have_encoder_and_decoder {
     $cmp_v = sprintf("%.2f", int($cmp_v*100)/100);
     my %compat_versions = map {$_ => 1} $Class->_test_compat();
     if (not defined $cmp_v or not exists $compat_versions{$cmp_v}) {
-        note("Could not load correct version of $need_class for testing "
+        diag("Could not load correct version of $need_class for testing "
              ."(got: $cmp_v, needed any of ".join(", ", keys %compat_versions).")");
         return();
     }
@@ -734,7 +734,7 @@ sub _get_roundtrip_tests {
             1023,1024,1025,
             8191,8192,8193,
         )),
-        ( map { [ ( $Sereal::Encoder::VERSION lt "3.003_002" ? "TODO" : "") 
+        ( map { [ ( $Sereal::Encoder::VERSION < 3.005002 ? "TODO" : "")
                     . " troublesome num/strs '$_'", $_ ] } @numstr ),
         ["long latin1 string", "üll" x 10000],
         ["long utf8 string", do {use utf8; " עדיין חשב" x 10000}],
@@ -1099,7 +1099,8 @@ sub run_roundtrip_tests_internal {
             eval {$decoded2 = $dec->($encoded2); 1}
                 or do {
                     my $err = $@ || 'Zombie error';
-                    diag("Got error while encoding the second time: $err");
+                    diag("Got error while decoding the second time: $err");
+                    # hobodecode($encoded2);
                 };
 
             defined($decoded2) == defined($data)
@@ -1127,7 +1128,7 @@ sub run_roundtrip_tests_internal {
             eval {$decoded3 = $dec->($encoded3); 1}
                 or do {
                     my $err = $@ || 'Zombie error';
-                    diag("Got error while encoding the third time: $err");
+                    diag("Got error while decoding the third time: $err");
                 };
 
             defined($decoded3) == defined($data)
